/*
 * License: Public Domain
 * written by Frank Benoit
 */
package dbus_d_javahelper;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;

import org.freedesktop.dbus.DirectConnection;

public class DBusDLoader {
	

	private DirectConnection dc;
	private final String dhelper;
	private final String address;
	private InputStream resourceStream = null;
	
	public DBusDLoader( String dhelper ){
		this.dhelper = dhelper;
		address = DirectConnection.createDynamicTCPSession();
	}
	
	public DBusDLoader( InputStream resourceStream, File helperLocation ){
		this( new File( helperLocation, "DHelper.exe" ).getAbsolutePath() );
		this.resourceStream = resourceStream;
	}
	
	public DirectConnection getConnection(){
		return dc;
	}
	
	public void ensure( boolean cond ){
		if( !cond ){
			throw new RuntimeException( "ensure failed" );
		}
	}

	public synchronized DirectConnection startup() throws Exception {
		
		if( resourceStream != null ){
			FileOutputStream ostr = new FileOutputStream( dhelper );
			int c, s = 0;
			byte[] data = new byte[0x10000];
			while((c=resourceStream.read(data)) > 0 ){
				ostr.write( data, 0, c );
				s++;
			}
			resourceStream.close();
			ostr.close();
			
			// set execute permission
			Process prc = Runtime.getRuntime().exec( new String[]{ "chmod", "u+x", dhelper });
			prc.waitFor();
		}
		new Thread() {
			@Override
			public void run() {
				try {
					ConsoleProcess process = new ConsoleProcess(
							new String[] {
									dhelper },
							new String[] {
									"DBUS_ADDRESS="+address },
							null,
							System.out,
							System.err);
					process.join();
				} catch (InterruptedException e) {
					e.printStackTrace();
				} catch (IOException e) {
					e.printStackTrace();
				}
			}
		}.start();

		dc = new DirectConnection(address + ",listen=true");
		return dc;
	}
	
	public synchronized void disconnect(){
		if( dc != null ){
			dc.disconnect();
			dc = null;
		}
	}
	
}
